#ifndef SCREEM_EDITOR_H
#define SCREEM_EDITOR_H

#include <sys/types.h>

#include "screem-view.h"

#define SCREEM_TYPE_EDITOR  (screem_editor_get_type())
#define SCREEM_EDITOR(obj)  (G_TYPE_CHECK_INSTANCE_CAST( ( obj ),   \
                                                SCREEM_TYPE_EDITOR, \
                                                ScreemEditor ) )
#define SCREEM_IS_EDITOR(obj)  (G_TYPE_CHECK_INSTANCE_TYPE( ( obj ), \
                                               SCREEM_TYPE_EDITOR ) )

typedef struct ScreemEditorPrivate ScreemEditorPrivate;

typedef struct {
	ScreemView parent;

	ScreemEditorPrivate *private;
} ScreemEditor;

typedef struct {
	ScreemViewClass parent_class;

	gchar (*request_context) ( ScreemEditor *editor, guint pos,
				   gchar **context_name, guint *depth,
				   guint *start, guint *end, gboolean build );

} ScreemEditorClass;

typedef enum {
	TEXT,
	ADD_CLOSE
} ScreemEditorAction;

typedef struct {
	gchar *key_name;
	gchar *text;
	ScreemEditorAction action;
} ScreemEditorMacro;

typedef gboolean (*KeysCallback)( ScreemEditor *editor,
				  GdkEventKey *event, 
				  ScreemEditorMacro *macro );

GType screem_editor_get_type( void );
ScreemEditor *screem_editor_new( ScreemWindow *window );

void screem_editor_insert( ScreemEditor *editor, gint pos, const gchar *text );
gboolean screem_editor_focus( ScreemEditor *editor );

guint screem_editor_get_length( ScreemEditor *editor );
gchar *screem_editor_get_text( ScreemEditor *editor, guint from, guint len );
gboolean screem_editor_has_selection( ScreemEditor *editor, guint *start,
				      guint *end );
void screem_editor_select_region( ScreemEditor *editor, guint start,
				  guint len );
void screem_editor_cut( ScreemEditor *editor );
void screem_editor_copy( ScreemEditor *editor );
void screem_editor_paste( ScreemEditor *editor );
void screem_editor_paste_encoded( ScreemEditor *editor );
void screem_editor_clear_selection( ScreemEditor *editor );
void screem_editor_set_pos_delayed( ScreemEditor *editor, guint pos );
void screem_editor_set_pos( ScreemEditor *editor, guint pos );
guint screem_editor_get_pos( ScreemEditor *editor );
void screem_editor_delete_forward( ScreemEditor *editor, guint pos, guint len);

void screem_editor_insert_markup( ScreemEditor *editor, 
				  const gchar *open_element, 
				  const gchar *close_element );
/* YUK */
void screem_editor_insert_attr_cb( GtkWidget *item, const gchar *attribute );
void screem_editor_insert_attribute( ScreemEditor *editor, 
				     const gchar *attribute );
void screem_editor_insert_file( ScreemEditor *editor, const gchar *filename );

void screem_editor_encode_text( ScreemEditor *editor );

gchar* screem_editor_get_word( gint pos, gchar *text );

void screem_editor_goto_line( ScreemEditor *editor, gint line );

void screem_editor_set_colours( ScreemEditor *editor );

void screem_editor_expose_lines( ScreemEditor *editor );

#endif
