#ifndef SCREEM_PAGE_H
#define SCREEM_PAGE_H

#include <gtk/gtkobject.h>
#include <gtk/gtktextbuffer.h>
#include <gtk/gtkliststore.h>
#include <gtk/gtktreemodel.h>

#include <glib.h>
#include <time.h>

#define SCREEM_TYPE_PAGE            (screem_page_get_type ())
#define SCREEM_PAGE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SCREEM_TYPE_PAGE, ScreemPage))
#define SCREEM_PAGE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SCREEM_TYPE_PAGE, ScreemPageClass))
#define SCREEM_IS_PAGE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SCREEM_TYPE_PAGE))

typedef struct ScreemPagePrivate ScreemPagePrivate;


typedef struct ScreemPageBookMark {
	gchar   *line_number;
	GString *name;
	gpointer iter;
} ScreemPageBookMark;

typedef struct  {
	GObject parent;

	ScreemPagePrivate *private;
} ScreemPage;

typedef struct {
        GObjectClass parent_class;

	/* emited when the page has changed */
	void (*changed) (ScreemPage *page, gboolean state);
	/* emited when the doctype has changed */
	void (*doctype_changed) (ScreemPage *page, const gchar *doctype );
	/* emited when the charset has changed */
	void (*charset_changed) (ScreemPage *page, const gchar *chartype );
	
	/* emited when the page is saved, this is so we can update
		other pages if this is a template etc. */
	void (*saved)(ScreemPage *page );
} ScreemPageClass;

ScreemPage *screem_page_new( void );
GType screem_page_get_type( void );

gboolean screem_page_is_loaded( ScreemPage *page );

gboolean screem_page_is_open( ScreemPage *page );
void screem_page_set_open( ScreemPage *page, gboolean opened );

void screem_page_set_pathname( ScreemPage *page, const gchar *path );
const gchar *screem_page_get_pathname( ScreemPage *page );

void screem_page_set_mime_type( ScreemPage *page, const gchar *mime_type );
const gchar *screem_page_get_mime_type( ScreemPage *page );

const gchar *screem_page_get_doctype( ScreemPage *page );

void screem_page_set_charset( ScreemPage *page, const gchar *charset );
const gchar *screem_page_get_charset( ScreemPage *page );

void screem_page_set_data( ScreemPage *page, const gchar *data );
gchar *screem_page_get_data( ScreemPage *page );

void screem_page_set_changed( ScreemPage *page, gboolean flag );
gboolean screem_page_get_changed( ScreemPage *page );

gboolean screem_page_read( ScreemPage *page, int fd );
gboolean screem_page_load( ScreemPage *page );
gboolean screem_page_revert( ScreemPage *page );
gboolean screem_page_save( ScreemPage *page );

gboolean screem_page_is_file_page( const gchar *path );
gboolean screem_page_is_mime_type_page( const gchar *mime_type );

gboolean screem_page_get_modified( ScreemPage *page );

/* exposed so ScreemEditor can get at it */
GtkTextBuffer *screem_page_get_text_buffer( ScreemPage *page );

/* needed for row activate callback on bookmarks dialog */
GtkListStore *screem_page_get_bookmark_model( ScreemPage *page );

void screem_page_add_bookmark( ScreemPage *page, gint line, const gchar *name );
void screem_page_remove_bookmark( ScreemPage *page, const gchar *name );
gint screem_page_get_bookmark( ScreemPage *page, const gchar *name );
const gchar* screem_page_bookmarked( ScreemPage *page, gint line );

gint screem_page_get_line_count( ScreemPage *page );

void screem_page_undo( ScreemPage *page );
void screem_page_redo( ScreemPage *page );


/* get a list of all mime types that a page can be (i.e all syntax
   file types) */
GSList *screem_page_get_mime_types( ScreemPage *page );

gboolean screem_page_is_dynamic( ScreemPage *page );

gboolean screem_page_is_template( ScreemPage *page, gchar **tag,
				gchar **path, GSList **blocks );

#endif
