/*  Screem:  php.c,
 *  PHP support, only really tooltips at the moment
 *
 *  Copyright (C) 2000  David A Knight
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  For contact information with the author of this source code please see
 *  the AUTHORS file.  If there is no AUTHORS file present then check the
 *  about box under the help menu for a contact address
 */

#include <config.h>

#include <string.h>

#include "php3.h"
#include "support.h"
#include "fileops.h"

#include "screem-editor.h"
#include "screem-site.h"
#include "screem-page.h"
#include "screem-dtd.h"
#include "screem-window.h"

static GHashTable *php_func_table;

void screem_php_build_function_hash_table()
{
	gchar *buffer;
	gchar *proto;
	gchar *name;
	gchar *temp;
	FILE *in;
	const gchar *func_file = DATADIR"/screem/php/php4_functions";

	php_func_table = g_hash_table_new(g_str_hash, g_str_equal);

	/* now place all php functions in the hash table,
	   the name of the function as the key, and the
	   prototype as the value */

	in = fopen( func_file, "r" );

	if( ! in )
		return;

	buffer = g_new( gchar, BUFSIZ );
	while( fgets( buffer, BUFSIZ, in ) ) {
		/* we have read in a line,
		   ignore the class the function belongs to 
		 */
		if( ( proto = strchr( buffer, '\n' ) ) ) {
			*proto = '\0';
		}

		if( ( proto = strchr( buffer, ':' ) ) ) {
			proto += 1;
			if( ( name = strchr( proto, ' ' ) ) ) {
				name += 1;
				if( ( temp = strchr( name, '(' ) ) ) {
					proto = g_strdup( proto );
					name = g_strndup( name, temp - name );
					g_hash_table_insert( php_func_table, 
							     name, 
							     proto );
				}
			}
		}
	}
	g_free( buffer );
}

void screem_php_destroy_function_hash_table()
{
	g_hash_table_destroy( php_func_table );
}

#if 0
void screem_php_authenticate()
{
	/* this is from the PHP3 manual */
	const gchar *auth = "\
<script language=\"php\">\n\
if(!isset($PHP_AUTH_USER)) {\n\
\tHeader(\"WWW-Authenticate: Basic realm=\\\"My Realm\\\"\");\n\
\tHeader(\"HTTP/1.0 401 Unauthorized\");\n\
\techo \"Text to send if user hits Cancel button\\n\";\n\
\texit;\n\
} else {\n\
\techo \"Hello $PHP_AUTH_USER.<P>\";\n\
\techo \"You entered $PHP_AUTH_PW as your password.<P>\";\n\
}\n\
</script>";
	screem_editor_insert( -1, auth );
}

void screem_php_require()
{
	gchar *filename;
	gchar *text;
	ScreemPage *page;
	gchar *path;
	gchar cwd[ 16384 ];
	const gchar *site_path;
	const gchar *require = "<?php require '%s' ?>";

	getcwd( cwd, 16384 );

	filename = bonobo_file_selector_open( NULL, FALSE,
					      _("Select file to include"),
					      NULL, NULL );

	if( ! filename )
		return;

	/* make filename relative to the page path */
	//	page = screem_site_get_current_page( current_site );
	site_path = screem_site_get_pathname( current_site );
	//	screem_page_chdir( page );
	path = relative_path( filename, site_path );
	g_free( filename );
     
	text = g_strdup_printf( require, path );

	//	screem_editor_insert( -1, text );

	g_free( path );
	g_free( text );

	chdir( cwd );
}
#endif
gboolean php_tooltip( ScreemEditor *editor, ScreemWindow *window,
		      gint pos, gchar *text )
{
	gchar *word;
	gchar *value;
	gboolean ret;
	ScreemDTD *dtd;

	ret = FALSE;

	/* FIXME: if pos is not inside a php tag / script context then
	   we don't want to do this */
	dtd = screem_window_get_current_dtd( window );

	/* get the word */
	word = screem_editor_get_word( pos, text );

	/* does word match a PHP function? 
	   what about a variable?

	   with a function we can show its prototype,
	   with a variable we could show its type and value */

	if( word ) {
		/* word obtained, is it a php function? */
		value = (gchar*)g_hash_table_lookup( php_func_table, word );
		if( value ) {
			/* it is */
			screem_window_show_message( window, value );
			ret = TRUE;
		}
		g_free( word );
	}

	return ret;
}
