/*  Screem:  screem-main.c
 *
 *  initialisation code, shouldn't do much as most of the init stuff
 *  belongs in screem-application.c
 *
 *  Copyright (C) 2001 David A Knight
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  For contact information with the author of this source code please see
 *  the AUTHORS file.  If there is no AUTHORS file present then check the
 *  about box under the help menu for a contact address
 */

#include <config.h>
#include <gmodule.h>

#include <libgnome/gnome-i18n.h>
#include <libgnome/gnome-config.h>
#include <libgnome/gnome-init.h>
#include <libgnome/gnome-util.h>
#include <libgnomeui/gnome-client.h>
#include <libgnomeui/gnome-ui-init.h>

#include <gtk/gtkmain.h>

#include <bonobo-activation/bonobo-activation-init.h>

#include <unistd.h>
#include <errno.h>

#include <glade/glade-init.h>
#include <glade/glade.h>
#include <sys/stat.h>

#include <gconf/gconf.h>

#include <libgnomevfs/gnome-vfs.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>

#include "pageUI.h"
#include "php3.h"
#include "fileops.h"

#include "screem-application.h"
#include "screem-site-ui.h"
#include "screem-page.h"

ScreemApplication *app;
GnomeClient *client = NULL;

static gint no_splash;

/* passed parameters stuff */
static const struct poptOption options[] = {
	{ "no-splash", '\0', POPT_ARG_NONE, &no_splash, 1,
	  N_("Disables splash screen on start up"), NULL },
        { NULL, '\0', 0, NULL, 0, NULL, NULL }
};

static gint
save_yourself( GnomeClient *client, gint phase, GnomeSaveStyle save_style,
               gint is_shutdown, GnomeInteractStyle interact_style,
               gint is_fast, gpointer client_data );
static gboolean screem_main_start_app( const gchar **start_files );
static const gchar** parse_args( poptContext ctx, int argc, char *argv[] );

int main( int argc, char *argv[] )
{
        CORBA_ORB orb;
	GnomeProgram *program;
	poptContext ctx;
	const gchar** start_files;

#ifdef ENABLE_NLS
	/* initialise i18n stuff */
        bindtextdomain( GETTEXT_PACKAGE, GNOMELOCALEDIR );
	bind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );
        textdomain( GETTEXT_PACKAGE );
#endif	

	program = gnome_program_init( PACKAGE, VERSION, LIBGNOMEUI_MODULE,
				      argc, argv,
				      GNOME_PROGRAM_STANDARD_PROPERTIES,
				      GNOME_PARAM_POPT_TABLE, options,
				      GNOME_PARAM_APP_DATADIR, DATADIR,
				      LIBGNOMEUI_PARAM_DEFAULT_ICON,
				      DATADIR"/pixmaps/screem.png",
				      GNOME_PARAM_HUMAN_READABLE_NAME,
				      _( "Site CReation and Editing EnvironMent" ),
				      NULL );

	gdk_threads_init();

        gnome_window_icon_set_default_from_file( DATADIR"/pixmaps/screem.png" );


	/* parse arguments */
	g_object_get( G_OBJECT( program ), GNOME_PARAM_POPT_CONTEXT, &ctx,
		      NULL );
	start_files = parse_args( ctx, argc, argv );

        orb = bonobo_activation_orb_init( &argc, argv );
        gnome_vfs_init();

	glade_require( "gnome" );

	g_idle_add( (GSourceFunc)screem_main_start_app, start_files );

	gdk_threads_enter();
	gtk_main();
	gdk_threads_leave();

	g_object_unref( app );

	gnome_vfs_shutdown();

	return 0;
}

static gboolean screem_main_start_app( const gchar **start_files )
{
	app = screem_application_new();

	screem_application_startup( app, start_files );

	return FALSE;
}

static const gchar **parse_args( poptContext ctx, int argc, char *argv[] )
{
	const gchar** start_files;

        /* passed arguements */
        start_files = poptGetArgs( ctx );

        /* Session Managment stuff */
        client = gnome_master_client ();
        g_signal_connect( G_OBJECT( client ), "save_yourself",
			  G_CALLBACK( save_yourself ), (gpointer)argv[0] );
        g_signal_connect( G_OBJECT( client ), "die",
			  G_CALLBACK( gtk_main_quit ), NULL );

	return start_files;
}

static gint
save_yourself( GnomeClient *client, gint phase, GnomeSaveStyle save_style,
               gint is_shutdown, GnomeInteractStyle interact_style,
               gint is_fast, gpointer client_data )
{
        gchar* args[4] = { "rm", "-r", NULL, NULL };

	args[2] = gnome_config_get_real_path
                    (gnome_client_get_config_prefix (client));
        gnome_client_set_discard_command (client, 3, args);

        return TRUE;
}

