/*  Screem:  screem-preferences.c
 *
 *  Copyright (C) 2002 David A Knight
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  For contact information with the author of this source code please see
 *  the AUTHORS file.  If there is no AUTHORS file present then check the
 *  about box under the help menu for a contact address
 */

#include <config.h>

#include <string.h>

#include <glade/glade.h>

#include <gconf/gconf-client.h>

#include <libgnome/gnome-i18n.h>

#include <libgnomeui/gnome-font-picker.h>
#include <libgnomeui/gnome-entry.h>
#include <libgnomeui/gnome-file-entry.h>
#include <libgnomeui/gnome-icon-entry.h>
#include <libgnomeui/gnome-color-picker.h>

#include <gtk/gtkentry.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtkspinbutton.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkcombo.h>
#include <gtk/gtkliststore.h>
#include <gtk/gtktreeview.h>
#include <gtk/gtktreeselection.h>
#include <gtk/gtkcellrenderer.h>
#include <gtk/gtkcellrenderertext.h>
#include <gtk/gtkframe.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkpaned.h>
#include <gtk/gtknotebook.h>

#include <gdk/gdkcolor.h>
#include <gdk/gdkkeysyms.h>

#include "screem-application.h"
#include "screem-preferences.h"
#include "screem-helper.h"

#include "rb-sidebar.h"
#include "rb-sidebar-button.h"

#include "support.h"

#include "xml.h"

#include "gnome-print-font-picker.h"

/* widget, gconf path */
static const gchar *toggle_prefs[] = {
	"autosite", "/apps/screem/autoload",
	"autopage", "/apps/screem/autoopen",
	"highlight", "/apps/screem/editor/highlight",
	"tagging", "/apps/screem/editor/inline_tagging",
	"indent", "/apps/screem/editor/auto_indent",
	"tagclose", "/apps/screem/editor/intelliclose",

	"browser1_static", "/apps/screem/Browser1_static",
	"browser2_static", "/apps/screem/Browser2_static",
	"browser3_static", "/apps/screem/Browser3_static",
	"browser1_dynamic", "/apps/screem/Browser1_dynamic",
	"browser2_dynamic", "/apps/screem/Browser2_dynamic",
	"browser3_dynamic", "/apps/screem/Browser3_dynamic",

	"print_headers", "/apps/screem/editor/print/headers",
	"wrap_lines", "/apps/screem/editor/print/wrap_lines",
	"print_lines", NULL,
	NULL
};

static const gchar *radio_prefs[] = {
	"autoenton", "0", "/apps/screem/editor/entityinsertion",
	"autoentoff", "1", "/apps/screem/editor/entityinsertion",
	"autoentbychar", "2", "/apps/screem/editor/entityinsertion",
	NULL
};

static const gchar *string_prefs[] = {
	"default_filename_entry", "/apps/screem/default_filename",
	"default_charset_combo", "/apps/screem/editor/default_charset",
	"default_dtd_combo", "/apps/screem/editor/default_dtd",
	"fontpicker", "/apps/screem/editor/font",
      	"browser1_path", "/apps/screem/Browser1_path",
	"browser2_path", "/apps/screem/Browser2_path",
	"browser3_path", "/apps/screem/Browser3_path",
	"browser1_icon", "/apps/screem/Browser1_icon",
	"browser2_icon", "/apps/screem/Browser2_icon",
	"browser3_icon", "/apps/screem/Browser3_icon",
	NULL
};

static const gchar *print_prefs[] = {
	"body_box", "/apps/screem/editor/print/body_font",
	"header_box", "/apps/screem/editor/print/header_font",
	"lines_box", "/apps/screem/editor/print/lines_font",
	NULL
};

static const gchar *spin_prefs[] = {
	"lines", "/apps/screem/editor/print/nlines",
	NULL
};

/* widget, key prefix, default */
static const gchar *editor_colours[] = {
	"editortextpicker", "/apps/screem/editor/text", "#ffffffffffff",
	"editorbgpicker", "/apps/screem/editor/back", "#000000000000",
	NULL
};

static const gchar *charsets[] = {
	"ISO-8859-1",
	"ISO-8859-2",
	"ISO-8859-5",
	"ISO-8859-7",
	"UTF-8",
	NULL
};

static void screem_preferences_edit_macros( ScreemApplication *application,
					    GladeXML *xml );
static void screem_preferences_edit_helpers( ScreemApplication *application,
					     GladeXML *xml );
static void screem_preferences_helper_select( GtkTreeSelection *sel );

static void screem_preferences_macro_select( GtkTreeSelection *sel );

static void screem_preferences_edit_syntax( ScreemApplication *application,
					    GladeXML *xml );
static void screem_preferences_build_syntax( const gchar *filename,
					     GObject *table,
					     GladeXML *xml );
void screem_preferences_tag_name_changed( GtkWidget *widget );
void screem_preferences_syntax_toggled( GtkWidget *widget );
void screem_preferences_syntax_fore_set( GnomeColorPicker *cp,
					 guint r, guint g, guint b,
					 guint a );
void screem_preferences_syntax_back_set( GnomeColorPicker *cp,
					 guint r, guint g, guint b,
					 guint a );

gboolean screem_preferences_toggle_changed( GtkWidget *toggle )
{
	ScreemApplication *application;
	gboolean state;
	const gchar *key;
	const gchar *name;

	name = gtk_widget_get_name( toggle );

	application = 
		SCREEM_APPLICATION( g_object_get_data( G_OBJECT( toggle ),
						       "application" ) );

	state = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) );
	
	if( strcmp( "print_lines", name ) ) {
		key = g_object_get_data( G_OBJECT( toggle ), "key" );
		
		gconf_client_set_bool( application->client, key, state, NULL );
	} else {
		GladeXML *xml;
		GtkWidget *spin;

		xml = glade_get_widget_tree( toggle );
		spin = glade_xml_get_widget( xml, "lines" );
		if( ! state ) {
			gtk_spin_button_set_value( GTK_SPIN_BUTTON( spin ),
						   0.0 );
		} else {
			gtk_spin_button_set_value( GTK_SPIN_BUTTON( spin ),
						   1.0 );
		}
	}

	return TRUE;
}

static void screem_preferences_toggle_notify( GConfClient *client,
					      guint cnxn_id,
					      GConfEntry *entry,
					      gpointer data )
{
	if( entry->value && entry->value->type == GCONF_VALUE_BOOL ) {
		GtkWidget *toggle;
		const gchar *name;
		gboolean wstate;
		gboolean state;
		
		toggle = GTK_WIDGET( data );

		name = gtk_widget_get_name( toggle );
		wstate = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
		state = gconf_value_get_bool( entry->value );

		if( wstate != state ) {
			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(toggle),
						      state );
		}
	}
}

void screem_preferences_spin_changed( GtkWidget *spin )
{
	ScreemApplication *application;
	const gchar *key;
	gint value;

	application = 
		SCREEM_APPLICATION( g_object_get_data( G_OBJECT( spin ),
						       "application" ) );
	key = g_object_get_data( G_OBJECT( spin ), "key" );
	
	value = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( spin ) );

	gconf_client_set_int( application->client, key, value, NULL );
}

static void screem_preferences_spin_notify( GConfClient *client,
					    guint cnxn_id,
					    GConfEntry *entry,
					    gpointer data )
{
	if( entry->value && entry->value->type == GCONF_VALUE_INT ) {
		GtkWidget *spin = GTK_WIDGET( data );
		gint value = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( spin ) );
		gint nvalue = gconf_value_get_int( entry->value );

		if( value != nvalue ) {
			gtk_spin_button_set_value( GTK_SPIN_BUTTON( spin ),
						   nvalue );
		}
	}
}

gboolean screem_preferences_icon_changed( GtkWidget *widget )
{
	ScreemApplication *application;
	const gchar *key;
	gchar *value;

	application = 
		SCREEM_APPLICATION( g_object_get_data( G_OBJECT( widget ),
						       "application" ) );
	key = g_object_get_data( G_OBJECT( widget ), "key" );
	
	value = gnome_icon_entry_get_filename( GNOME_ICON_ENTRY( widget ) );
	if( ! value )
		value = g_strdup( "" );

	gconf_client_set_string( application->client, key, value, NULL );

	g_free( value );

	return TRUE;
}

gboolean screem_preferences_string_change( GtkWidget *widget )
{
	GtkWidget *entry = NULL;
	const gchar *cvalue = NULL;
	const gchar *key;

	ScreemApplication *application;

       	application = 
		SCREEM_APPLICATION( g_object_get_data( G_OBJECT( widget ),
						       "application" ) );
	key = g_object_get_data( G_OBJECT( widget ), "key" );

	if( GNOME_IS_ENTRY( widget ) ) {
		entry = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		cvalue = gtk_entry_get_text( GTK_ENTRY( entry ) );
	} else if( GTK_IS_COMBO( widget ) ) {
		entry = GTK_COMBO( widget )->entry;
		cvalue = gtk_entry_get_text( GTK_ENTRY( entry ) );
	} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
		entry = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
		cvalue = gtk_entry_get_text( GTK_ENTRY( entry ) );	
	}
	    
	if( cvalue ) {
		gconf_client_set_string( application->client, key, 
					 cvalue, NULL );
	}

	return TRUE;
}

static void screem_preferences_string_notify( GConfClient *client,
					      guint cnxn_id,
					      GConfEntry *entry,
					      gpointer data )
{
	if( entry->value && entry->value->type == GCONF_VALUE_STRING ) {
		const gchar *value = gconf_value_get_string( entry->value );
		GtkWidget *widget;
		GtkWidget *gtkentry = NULL;

		widget = GTK_WIDGET( data );

		if( GNOME_IS_ENTRY( widget ) ) {
			gtkentry = gnome_entry_gtk_entry( GNOME_ENTRY(widget));
		} else if( GTK_IS_ENTRY( widget ) ) {
			gtkentry = widget;
		} else if( GTK_IS_COMBO( widget ) ) {
			gtkentry = GTK_COMBO( widget )->entry;
		} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
			gtkentry = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
		} else if( GNOME_IS_ICON_ENTRY( widget ) ) {
			const gchar *cvalue;
			cvalue = gnome_icon_entry_get_filename( GNOME_ICON_ENTRY( widget ) );
			if( ! cvalue || strcmp( cvalue, value ) )
				gnome_icon_entry_set_filename( GNOME_ICON_ENTRY( widget ),
							      value);
		} else if( GNOME_PRINT_IS_FONT_PICKER( widget ) ) {
			const gchar *cvalue;
			cvalue = gnome_print_font_picker_get_font_name( GNOME_PRINT_FONT_PICKER( widget ) );
			if( ! cvalue || strcmp( cvalue, value ) )
				gnome_print_font_picker_set_font_name( GNOME_PRINT_FONT_PICKER( widget ), value );
		} else if( GNOME_IS_FONT_PICKER( widget ) ) {
			const gchar *cvalue;
			cvalue = gnome_font_picker_get_font_name( GNOME_FONT_PICKER( widget ) );
			if( ! cvalue || strcmp( cvalue, value ) )
				gnome_font_picker_set_font_name( GNOME_FONT_PICKER( widget ), value );
			
		}
		if( gtkentry ) {
			const gchar *cvalue;
			cvalue = gtk_entry_get_text( GTK_ENTRY( gtkentry ) );
			if( value && strcmp( value, cvalue ) ) {
				gtk_entry_set_text( GTK_ENTRY( gtkentry ),
						    value );
			}
		}
	}
}

void screem_preferences_font_set( GtkWidget *fp,
				  const gchar *font )
{
	ScreemApplication *application;
	const gchar *key;

	application = g_object_get_data( G_OBJECT( fp ), "application" );
	key = g_object_get_data( G_OBJECT( fp ), "key" );
	gconf_client_set_string( application->client, key, font, NULL );
}

void screem_preferences_editor_colour_set( GnomeColorPicker *cp,
					   guint r, guint g, guint b,
					   guint a )
{
	ScreemApplication *application;
	const gchar *key;
	gchar *value;

	application = g_object_get_data( G_OBJECT( cp ), "application" );
	key = g_object_get_data( G_OBJECT( cp ), "key" );
	value = g_strdup_printf( "#%.4x%.4x%.4x", r, g, b );
	gconf_client_set_string( application->client, key, value, NULL );
	g_free( value );
}

static void screem_preferences_editor_colour_notify( GConfClient *client,
						     guint cnxn_id,
						     GConfEntry *entry,
						     gpointer data )
{
	if( entry->value && entry->value->type == GCONF_VALUE_STRING ) {
		const gchar *value = gconf_value_get_string( entry->value );
		GdkColor colour;
		GtkWidget *widget;
		GdkColor pcolour;
		gushort junk;
	
		widget = GTK_WIDGET( data );
		gnome_color_picker_get_i16( GNOME_COLOR_PICKER( widget ),
				    &pcolour.red, &pcolour.green, 
				    &pcolour.blue, &junk );

		if( gdk_color_parse( value, &colour ) && 
		    ! gdk_color_equal( &colour, &pcolour ) ) {
			gnome_color_picker_set_i16( GNOME_COLOR_PICKER(widget),
						    colour.red, colour.green,
						    colour.blue, 0 );
		}
	}
}

gboolean screem_preferences_radio_changed( GtkWidget *toggle )
{
	ScreemApplication *application;
	gboolean state;
	gint val;
	const gchar *key;
	const gchar *name;

	name = gtk_widget_get_name( toggle );

	application = 
		SCREEM_APPLICATION( g_object_get_data( G_OBJECT( toggle ),
						       "application" ) );
	val = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( toggle ),
						  "val" ) );
	state = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) );
	
	key = g_object_get_data( G_OBJECT( toggle ), "key" );
	
	if( state ) {
		gconf_client_set_int( application->client, key, val, NULL );
	}
	
	return TRUE;
}

static void screem_preferences_radio_toggle_notify( GConfClient *client,
						    guint cnxn_id,
						    GConfEntry *entry,
						    gpointer data )

{
	if( entry->value && entry->value->type == GCONF_VALUE_INT ) {
		GtkWidget *toggle;
		const gchar *name;
		gboolean wstate;
		gint gval;
		gint val;
		
		toggle = GTK_WIDGET( data );
		name = gtk_widget_get_name( toggle );

		wstate= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
		gval = gconf_value_get_bool( entry->value );
		
		val = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( toggle ),
							  "val" ) );

		if( val != gval && wstate ) {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle),
						     FALSE );
		} else if( val == gval && ! wstate ) {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle),
						     TRUE );
		}
	}
}

static void screem_preferences_switch_tab( GtkWidget *widget, GladeXML *xml )
{
	gint tab;

	tab = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( widget ),
						   "tab" ) );

	widget = glade_xml_get_widget( xml, "notebook" );
	gtk_notebook_set_current_page( GTK_NOTEBOOK( widget ), tab );
}

void screem_preferences_edit( ScreemApplication *application )
{
	GConfClient *client;
	gchar *gladepath;
	GladeXML *xml;
	GtkWidget *widget;
	GtkWidget *pane;
	GtkWidget *button;
	gint i;
	GtkWidget *dialog;
	GList *list;

	const gchar *catagories[] = {
		"Misc", "Browsers",
		"Macros", "Helpers",
		"Highlighting", "Printing",
		NULL
	};

	client = application->client;

	/* create dialog */
	gladepath = gconf_client_get_string( client, 
					     "/apps/screem/general/glade_path",
					     NULL );
	xml = glade_xml_new( gladepath, "preferences", NULL );
	g_free( gladepath );

	pane = glade_xml_get_widget( xml, "left" );

	widget = GTK_WIDGET( rb_sidebar_new() );

	for( i = 0; catagories[ i ]; ++ i ) {
		button = GTK_WIDGET( rb_sidebar_button_new( catagories[ i ],
							    catagories[ i ] ) );
		rb_sidebar_button_set( RB_SIDEBAR_BUTTON( button ), "Screem_Site",
				       catagories[ i ], TRUE );
		g_object_set_data( G_OBJECT( button ), "tab", 
				   GINT_TO_POINTER( i ) );
		g_signal_connect( G_OBJECT( button ), "clicked",
				  G_CALLBACK( screem_preferences_switch_tab ),
				  xml );
		rb_sidebar_append( RB_SIDEBAR( widget ), RB_SIDEBAR_BUTTON( button ) );
		gtk_widget_show( button );
	}

	gtk_box_pack_start( GTK_BOX( pane ), widget, TRUE, TRUE, 0 );
	gtk_widget_show_all( pane );
       
	widget = glade_xml_get_widget( xml, "notebook" );
	gtk_notebook_set_current_page( GTK_NOTEBOOK( widget ), 0 );

	gtk_notebook_set_show_tabs( GTK_NOTEBOOK( widget ), FALSE );


	/* setup printing font pickers */
	for( i = 0; print_prefs[ i ]; i += 2 ) {
		gchar *value;
		gint handle;

		widget = glade_xml_get_widget( xml, 
					       print_prefs[ i ] );
		button = gnome_print_font_picker_new();
		gnome_print_font_picker_set_mode( GNOME_PRINT_FONT_PICKER( button ),
						  GNOME_PRINT_FONT_PICKER_MODE_FONT_INFO );
		gnome_print_font_picker_fi_set_use_font_in_label( GNOME_PRINT_FONT_PICKER( button ),
							    TRUE, 14 );
		gnome_print_font_picker_fi_set_show_size( GNOME_PRINT_FONT_PICKER( button ),
						    TRUE );

		gtk_widget_show( button );
		gtk_box_pack_start( GTK_BOX( widget ), button, TRUE, TRUE, 0 );

	
		g_signal_connect( G_OBJECT( button ), "font_set",
				  G_CALLBACK( screem_preferences_font_set ),
				  NULL );

		value = gconf_client_get_string( client, print_prefs[ i + 1 ],
						 NULL );
		if( ! value ) {
			value = g_strdup( "" );
		}
		gnome_print_font_picker_set_font_name( GNOME_PRINT_FONT_PICKER( button ), value );
		g_object_set_data( G_OBJECT( button ), "application",
				   application );
		g_object_set_data( G_OBJECT( button ), "key",
				   (gchar*)print_prefs[ i + 1 ] );
		handle = gconf_client_notify_add( client,
						  print_prefs[ i + 1 ],
						  screem_preferences_string_notify, 
						  button, NULL, NULL );
		g_object_set_data( G_OBJECT( button ), "notify",
				   GINT_TO_POINTER( handle ) );
	}


	/* setup defaults */
	widget = glade_xml_get_widget( xml, "default_dtd_combo" );
	screem_dtd_fill_combo( application, widget );
	
	widget = glade_xml_get_widget( xml, "default_charset_combo" );
	for( list = NULL, i = 0; charsets[ i ]; ++ i ) {
		list = g_list_prepend( list, (gpointer)charsets[ i ] );
	}
	list = g_list_reverse( list );
	gtk_combo_set_popdown_strings( GTK_COMBO( widget ), list );
	g_list_free( list );
	
	/* handle string preferences */
	for( i = 0; string_prefs[ i ]; i += 2 ) {
		GtkWidget *entry = NULL;
		gchar *value;
		gint handle;

		widget = glade_xml_get_widget( xml, string_prefs[ i ] );
		value = gconf_client_get_string( client, string_prefs[ i + 1 ],
						 NULL );
		if( ! value ) {
			value = g_strdup( "" );
		}
		if( GNOME_IS_ENTRY( widget ) ) {
			entry = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		} else if( GTK_IS_ENTRY( widget ) ) {
			entry = widget;
		} else if( GTK_IS_COMBO( widget ) ) {
			entry = GTK_COMBO( widget )->entry;
		} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
			entry = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
		} else if( GNOME_IS_ICON_ENTRY( widget ) ) {
			gnome_icon_entry_set_filename(GNOME_ICON_ENTRY(widget),
						      value);
		} else if( GNOME_IS_FONT_PICKER( widget ) ) {
			gnome_font_picker_set_font_name( GNOME_FONT_PICKER( widget ), value );
		}
		if( entry ) {
			gtk_entry_set_text( GTK_ENTRY( entry ), value );
		}
		g_free( value );

		g_object_set_data( G_OBJECT( widget ), "application",
				   application );
		g_object_set_data( G_OBJECT( widget ), "key",
				   (gchar*)string_prefs[ i + 1 ] );
		handle = gconf_client_notify_add( client,
						  string_prefs[ i + 1 ],
						  screem_preferences_string_notify, 
						  widget, NULL, NULL );
		g_object_set_data( G_OBJECT( widget ), "notify",
				   GINT_TO_POINTER( handle ) );
	}

	/* handle spin preferences */
	for( i = 0; spin_prefs[ i ]; i += 2 ) {
		GtkWidget *spin;
		gboolean value;
		gint handle;

		spin = glade_xml_get_widget( xml, spin_prefs[ i ] );
		value = gconf_client_get_int( client, spin_prefs[ i + 1 ],
					      NULL );
		gtk_spin_button_set_value( GTK_SPIN_BUTTON( spin ),
					   (gdouble)value );
		g_object_set_data( G_OBJECT( spin ), "application",
				   application );
		g_object_set_data( G_OBJECT( spin ), "key",
				   (gchar*)spin_prefs[ i + 1 ] );
		
		if( spin_prefs[ i + 1 ] ) {
			handle = gconf_client_notify_add( client,
							  spin_prefs[ i + 1 ],
							  screem_preferences_spin_notify,
							  spin, NULL, NULL );
			g_object_set_data( G_OBJECT( spin ), "notify",
					   GINT_TO_POINTER( handle ) );
		}
	}

	/* handle boolean preferences */
	for( i = 0; toggle_prefs[ i ]; i += 2 ) {
		GtkWidget *toggle;
		gboolean value;
		gint handle;

		toggle = glade_xml_get_widget( xml, toggle_prefs[ i ] );
		if( toggle_prefs[ i + 1 ] ) {
			value = gconf_client_get_bool( client, 
						       toggle_prefs[ i + 1 ],
						       NULL );
		} else if( ! strcmp( toggle_prefs[ i ], "print_lines" ) ) {
			GtkWidget *spin;
			gint val;
			spin = glade_xml_get_widget( xml, "lines" );
			val = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( spin ) );
			value = ( val != 0 );
		} else {
			value = FALSE;
		}
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( toggle ),
					      value );
		g_object_set_data( G_OBJECT( toggle ), "application",
				   application );
		g_object_set_data( G_OBJECT( toggle ), "key",
				   (gchar*)toggle_prefs[ i + 1 ] );
		
		if( toggle_prefs[ i + 1 ] ) {
			handle = gconf_client_notify_add( client,
							  toggle_prefs[ i + 1 ],
							  screem_preferences_toggle_notify, 
							  toggle, NULL, NULL );
		g_object_set_data( G_OBJECT( toggle ), "notify",
				   GINT_TO_POINTER( handle ) );
		}
	}
	
	/* handle editor colours */
	for( i = 0; editor_colours[ i ]; i += 3 ) {
		GtkWidget *widget;
		GdkColor colour;
		gchar *value;
		gint handle;

		widget = glade_xml_get_widget( xml, editor_colours[ i ] );
	       
		value = gconf_client_get_string( client, 
						 editor_colours[ i + 1 ],
						 NULL );
		if( ! value )
			value = g_strdup( editor_colours[ i + 2 ] );
		if( ! gdk_color_parse( value, &colour ) ) {
			/* corrupt config, use defaults */
			gdk_color_parse( editor_colours[ i + 2 ], &colour );
		}
		g_free( value );

		gnome_color_picker_set_i16( GNOME_COLOR_PICKER( widget ),
					    colour.red, colour.green,
					    colour.blue, 0 );

		g_object_set_data( G_OBJECT( widget ) , "application",
				   application );
		g_object_set_data( G_OBJECT( widget ), "key", 
				   (gchar*)editor_colours[ i + 1 ] );

		handle = gconf_client_notify_add( client,
						  editor_colours[ i + 1 ],
						  screem_preferences_editor_colour_notify,
						  widget, NULL, NULL );
		g_object_set_data( G_OBJECT( widget ), "notify",
				   GINT_TO_POINTER( handle ) );
	}

	/* setup radio buttons */
	for( i = 0; radio_prefs[ i ]; i += 3 ) {
		GtkWidget *toggle;
		gboolean value = 0;
		gint val;
		gint handle;
		
		toggle = glade_xml_get_widget( xml, radio_prefs[ i ] );
		val = atoi( radio_prefs[ i + 1 ] );
		if( radio_prefs[ i + 2 ] ) {
			value = gconf_client_get_int( client, 
						      radio_prefs[ i + 2 ],
						      NULL );
		}
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( toggle ),
					      val == value );
		g_object_set_data( G_OBJECT( toggle ), "application",
				   application );
		g_object_set_data( G_OBJECT( toggle ), "val",
				   GINT_TO_POINTER( val ) );
		g_object_set_data( G_OBJECT( toggle ), "key", 
				   (gchar*)radio_prefs[ i + 2 ] );
		
		handle = gconf_client_notify_add( client,
						  radio_prefs[ i + 2 ],
						  screem_preferences_radio_toggle_notify,
						  toggle, NULL, NULL );
		g_object_set_data( G_OBJECT( toggle ), "notify",
				   GINT_TO_POINTER( handle ) );
	}
	

	/* handle macros */
	screem_preferences_edit_macros( application, xml );

	/* handle helpers */
	screem_preferences_edit_helpers( application, xml );

	/* handle syntax files */
	screem_preferences_edit_syntax( application, xml );

	glade_xml_signal_autoconnect( xml );

	dialog = glade_xml_get_widget( xml, "preferences" );
	gtk_dialog_run( GTK_DIALOG( dialog ) );

	/* remove all notifications */
	for( i = 0; print_prefs[ i ]; i += 2 ) {
		GtkWidget *widget;
		gint handle;

		widget = glade_xml_get_widget( xml, print_prefs[ i ] );
		handle = 
			GPOINTER_TO_INT( g_object_get_data( G_OBJECT( widget ),
							    "notify" ) );
		if( print_prefs[ i + 1 ] ) {
			gconf_client_notify_remove( client, handle );
		}
	}
	for( i = 0; toggle_prefs[ i ]; i += 2 ) {
		GtkWidget *toggle;
		gint handle;

		toggle = glade_xml_get_widget( xml, toggle_prefs[ i ] );
		handle = 
			GPOINTER_TO_INT( g_object_get_data( G_OBJECT( toggle ),
							    "notify" ) );
		if( toggle_prefs[ i + 1 ] ) {
			gconf_client_notify_remove( client, handle );
		}
	}
	for( i = 0; spin_prefs[ i ]; i += 2 ) {
		GtkWidget *widget;
		gint handle;

		widget = glade_xml_get_widget( xml, spin_prefs[ i ] );
		handle = 
			GPOINTER_TO_INT( g_object_get_data( G_OBJECT( widget ),
							    "notify" ) );
		if( spin_prefs[ i + 1 ] ) {
			gconf_client_notify_remove( client, handle );
		}
	}

	for( i = 0; radio_prefs[ i ]; i += 3 ) {
		GtkWidget *toggle;
		gint handle;

		toggle = glade_xml_get_widget( xml, radio_prefs[ i ] );
		handle = 
			GPOINTER_TO_INT( g_object_get_data( G_OBJECT( toggle ),
							    "notify" ) );
		if( radio_prefs[ i + 2 ] ) {
			gconf_client_notify_remove( client, handle );
		}
		
	}
	for( i = 0; string_prefs[ i ]; i += 2 ) {
		GtkWidget *widget;
		gint handle;

		widget = glade_xml_get_widget( xml, string_prefs[ i ] );
		handle = 
			GPOINTER_TO_INT( g_object_get_data( G_OBJECT( widget ),
							    "notify" ) );
		gconf_client_notify_remove( client, handle );
	}
	for( i = 0; editor_colours[ i ]; i += 3 ) {
		GtkWidget *widget;
		gint handle;

		widget = glade_xml_get_widget( xml, editor_colours[ i ] );
		handle = 
			GPOINTER_TO_INT( g_object_get_data( G_OBJECT( widget ),
							    "notify" ) );
		gconf_client_notify_remove( client, handle );
	}
	gtk_widget_destroy( dialog );
}


static void screem_preferences_edit_macros( ScreemApplication *application,
					    GladeXML *xml )
{
	GtkListStore *store;
	GList *macros;

	GtkWidget *view;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *col;
	const gchar *titles[] = { "Key Combo", "Action/Text", NULL };
	gint i;
	GtkTreeSelection *selection;

	store = gtk_list_store_new( 3, G_TYPE_STRING, G_TYPE_STRING,
				    G_TYPE_POINTER, NULL );
	/* build model */
	for( macros = application->macros; macros; macros = macros->next ) {
		GtkTreeIter it;
		ScreemEditorMacro *macro;

		macro = (ScreemEditorMacro*)macros->data;

		gtk_list_store_append( store, &it );
		gtk_list_store_set( store, &it, 
				    0, macro->key_name,
				    1, macro->text,
				    2, macro, -1 );
	}

	/* create tree view */
	view = glade_xml_get_widget( xml, "macrolist" );
	g_object_set_data( G_OBJECT( view ) , "application", application );

	for( i = 0; titles[ i ]; ++ i ) {
		renderer = gtk_cell_renderer_text_new();
		col = gtk_tree_view_column_new();
		gtk_tree_view_column_set_title( col, titles[ i ] );
		gtk_tree_view_column_pack_start( col, renderer, TRUE );
		gtk_tree_view_append_column( GTK_TREE_VIEW( view ), col );
		gtk_tree_view_column_set_attributes( col, renderer, 
						     "text", i, NULL );
	}
	
	gtk_tree_view_set_model( GTK_TREE_VIEW( view ), 
				 GTK_TREE_MODEL( store ) );
	g_object_unref( store );

	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
	g_signal_connect( G_OBJECT( selection ), "changed",
			  G_CALLBACK( screem_preferences_macro_select ),
			  NULL );
}

/* also used for the update button as we perform a check to see
   if a helper already exists */
void screem_preferences_add_macro( GtkWidget *widget )
{
	GladeXML *xml;
	GtkWidget *view;
	ScreemEditorMacro *macro;
	ScreemApplication *application;
	GtkTreeModel *model;
	GtkTreeIter it;

	const gchar *combo;
	const gchar *action;

	xml = glade_get_widget_tree( widget );

	view = glade_xml_get_widget( xml, "macrolist" );
	model = gtk_tree_view_get_model( GTK_TREE_VIEW( view ) );

	application = SCREEM_APPLICATION( g_object_get_data( G_OBJECT( view ),
							     "application" ) );

	widget = glade_xml_get_widget( xml, "macrocombo" );
	combo = gtk_entry_get_text( GTK_ENTRY( widget ) );

	widget = glade_xml_get_widget( xml, "macroaction" );
	widget = GTK_COMBO( widget )->entry;
	action = gtk_entry_get_text( GTK_ENTRY( widget ) );

	macro = g_new0( ScreemEditorMacro, 1 );
	macro->key_name = g_strdup( combo );
	macro->text = g_strdup( action );

	screem_application_add_macro( application, macro );

	gtk_list_store_append( GTK_LIST_STORE( model ), &it );
	gtk_list_store_set( GTK_LIST_STORE( model ), &it, 
			    0, macro->key_name,
			    1, macro->text,
			    2, macro, -1 );
}

void screem_preferences_remove_macro( GtkWidget *widget )
{
	GladeXML *xml;
	GtkWidget *view;
	ScreemEditorMacro *macro;
	ScreemApplication *application;
	GtkTreeModel *model;
	GtkTreeIter it;
	GtkTreeSelection *selection;

	const gchar *combo;

	xml = glade_get_widget_tree( widget );

	view = glade_xml_get_widget( xml, "macrolist" );
	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );

	application = SCREEM_APPLICATION( g_object_get_data( G_OBJECT( view ),
							     "application" ) );

	if( gtk_tree_selection_get_selected( selection, &model, &it ) ) {

		widget = glade_xml_get_widget( xml, "macrocombo" );
		combo = gtk_entry_get_text( GTK_ENTRY( widget ) );
		
		macro = g_new0( ScreemEditorMacro, 1 );
		macro->key_name = (gchar*)combo;
		screem_application_remove_macro( application, macro );
		g_free( macro );

		gtk_list_store_remove( GTK_LIST_STORE( model ), &it );
	}
}

static void screem_preferences_macro_select( GtkTreeSelection *sel )
{
	GtkTreeView *view;
	GtkTreeModel *model;
	GtkTreeIter it;
	GtkWidget *widget;
	GladeXML *xml;

    	view = gtk_tree_selection_get_tree_view( sel );

	xml = glade_get_widget_tree( GTK_WIDGET( view ) );

	if( gtk_tree_selection_get_selected( sel, &model, &it ) ) {
		GValue nvalue = {0};
		GValue pvalue = {0};
		GtkWidget *widget;

		gtk_tree_model_get_value( model, &it, 0, &nvalue );
		gtk_tree_model_get_value( model, &it, 1, &pvalue );
		
		widget = glade_xml_get_widget( xml, "macrocombo" );
		gtk_entry_set_text( GTK_ENTRY( widget ),
				    g_value_get_string( &nvalue ) );

		widget = glade_xml_get_widget( xml, "macroaction" );
		widget = GTK_COMBO( widget )->entry;
		gtk_entry_set_text( GTK_ENTRY( widget ),
				    g_value_get_string( &pvalue ) );

		g_value_unset( &nvalue );
		g_value_unset( &pvalue );
	} else {
		widget = glade_xml_get_widget( xml, "macrocombo" );
		gtk_entry_set_text( GTK_ENTRY( widget ), "" );
		widget = glade_xml_get_widget( xml, "macroaction" );
		widget = GTK_COMBO( widget )->entry;
		gtk_entry_set_text( GTK_ENTRY( widget ), "" );
	}
}

gboolean screem_preferences_grab_key( GtkWidget *widget, GdkEventKey *event )
{
	gboolean ret;
	gchar *key;

	ret = FALSE;

	if( event->keyval != GDK_Tab &&
	    event->keyval != GDK_Up &&
	    event->keyval != GDK_Right &&
	    event->keyval != GDK_Down &&
	    event->keyval != GDK_Left ) {
		key = convert_keysym_state_to_string (event->keyval,
						      event->state);
		if( key ) {
			gtk_entry_set_text (GTK_ENTRY(widget), key );
			g_free( key );
		} else {
			gtk_entry_set_text (GTK_ENTRY(widget), "" );
		}
		ret = TRUE;
	}

	return ret;
}

static void screem_preferences_edit_helpers( ScreemApplication *application,
					     GladeXML *xml )
{
	GtkListStore *store;
	GList *helpers;
	GtkWidget *view;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *col;
	GtkTreeSelection *selection;
	const gchar *titles[] = { "Name", "Path", "Icon", NULL };
	gint i;


	store = gtk_list_store_new( 4, 
				    G_TYPE_STRING,
				    G_TYPE_STRING, G_TYPE_STRING,
				    G_TYPE_POINTER,
				    NULL );
	/* build model */
	for(helpers = application->helpers; helpers; helpers = helpers->next) {
		GtkTreeIter it;
		ScreemHelper *helper;
		const gchar *name;
		const gchar *pathname;
		const gchar *icon;

		helper = SCREEM_HELPER( helpers->data );

		g_object_get( G_OBJECT( helper ), 
			      "name", &name, "pathname", &pathname,
			      "iconpath", &icon, NULL );
		
		gtk_list_store_append( store, &it );

		gtk_list_store_set( store, &it,
				    0, name, 
				    1, pathname,
				    2, icon,
				    -1 );
	}

	/* create tree view */
	view = glade_xml_get_widget( xml, "helperlist" );
	g_object_set_data( G_OBJECT( view ) , "application", application );

	for( i = 0; titles[ i ]; ++ i ) {
		renderer = gtk_cell_renderer_text_new();
		col = gtk_tree_view_column_new();
		gtk_tree_view_column_set_title( col, titles[ i ] );
		gtk_tree_view_column_pack_start( col, renderer, TRUE );
		gtk_tree_view_append_column( GTK_TREE_VIEW( view ), col );
		gtk_tree_view_column_set_attributes( col, renderer, 
						     "text", i, NULL );
	}
	
	gtk_tree_view_set_model( GTK_TREE_VIEW( view ), 
				 GTK_TREE_MODEL( store ) );
	g_object_unref( store );

	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
	g_signal_connect( G_OBJECT( selection ), "changed",
			  G_CALLBACK( screem_preferences_helper_select ),
			  NULL );
}

/* also used for the update button as we perform a check to see
   if a helper already exists */
void screem_preferences_add_helper( GtkWidget *widget )
{
	GladeXML *xml;
	GtkWidget *view;
	ScreemHelper *helper;
	ScreemApplication *application;
	GtkTreeModel *model;
	GtkTreeIter it;

	const gchar *name;
	const gchar *pathname;
	gchar *iconpath;

	xml = glade_get_widget_tree( widget );

	view = glade_xml_get_widget( xml, "helperlist" );
	model = gtk_tree_view_get_model( GTK_TREE_VIEW( view ) );

	application = SCREEM_APPLICATION( g_object_get_data( G_OBJECT( view ),
							     "application" ) );

	widget = glade_xml_get_widget( xml, "helpername" );
	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
	name = gtk_entry_get_text( GTK_ENTRY( widget ) );

	widget = glade_xml_get_widget( xml, "helperpath" );
	widget = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
	pathname = gtk_entry_get_text( GTK_ENTRY( widget ) );

	widget = glade_xml_get_widget( xml, "helpericon" );
	iconpath = gnome_icon_entry_get_filename( GNOME_ICON_ENTRY( widget ) );

	helper = screem_helper_new( name, pathname, iconpath,
				    SCREEM_HELPER_STDIN_PAGE,
				    SCREEM_HELPER_STDOUT_PAGE );

	screem_application_add_helper( application, helper );

	gtk_list_store_append( GTK_LIST_STORE( model ), &it );
	gtk_list_store_set( GTK_LIST_STORE( model ), &it,
			    0, name, 
			    1, pathname,
			    2, iconpath,
			    -1 );

	g_free( iconpath );
}

void screem_preferences_remove_helper( GtkWidget *widget )
{
	GladeXML *xml;
	GtkWidget *view;
	ScreemHelper *helper;
	ScreemApplication *application;
	GtkTreeModel *model;
	GtkTreeIter it;
	GtkTreeSelection *selection;

	const gchar *name;

	xml = glade_get_widget_tree( widget );

	view = glade_xml_get_widget( xml, "helperlist" );
	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );

	application = SCREEM_APPLICATION( g_object_get_data( G_OBJECT( view ),
							     "application" ) );
	
	if( gtk_tree_selection_get_selected( selection, &model, &it ) ) {
		widget = glade_xml_get_widget( xml, "helpername" );
		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		name = gtk_entry_get_text( GTK_ENTRY( widget ) );
		
		helper = screem_helper_new( name, "", "",
					    SCREEM_HELPER_STDIN_PAGE,
					    SCREEM_HELPER_STDOUT_PAGE );
		screem_application_remove_helper( application, helper );

		gtk_list_store_remove( GTK_LIST_STORE( model ), &it );

		g_object_unref( G_OBJECT( helper ) );
	}
}

static void screem_preferences_helper_select( GtkTreeSelection *sel )
{
	GtkTreeView *view;
	GtkTreeModel *model;
	GtkTreeIter it;
	GtkWidget *widget;
	GladeXML *xml;

       	view = gtk_tree_selection_get_tree_view( sel );

	xml = glade_get_widget_tree( GTK_WIDGET( view ) );

	if( gtk_tree_selection_get_selected( sel, &model, &it ) ) {
		GValue nvalue = {0};
		GValue pvalue = {0};
		GValue ivalue = {0};
		GladeXML *xml;
		GtkWidget *widget;

		gtk_tree_model_get_value( model, &it, 0, &nvalue );
		gtk_tree_model_get_value( model, &it, 1, &pvalue );
		gtk_tree_model_get_value( model, &it, 2, &ivalue );
		
		xml = glade_get_widget_tree( GTK_WIDGET( view ) );
		
		widget = glade_xml_get_widget( xml, "helpername" );
		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		gtk_entry_set_text( GTK_ENTRY( widget ),
				    g_value_get_string( &nvalue ) );

		widget = glade_xml_get_widget( xml, "helperpath" );
		widget = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(widget));
		gtk_entry_set_text( GTK_ENTRY( widget ),
				    g_value_get_string( &pvalue ) );

		widget = glade_xml_get_widget( xml, "helpericon" );
		gnome_icon_entry_set_filename( GNOME_ICON_ENTRY( widget ),
					       g_value_get_string( &ivalue ) );

		g_value_unset( &nvalue );
		g_value_unset( &pvalue );
		g_value_unset( &ivalue );
	} else {
		widget = glade_xml_get_widget( xml, "helpername" );
		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		gtk_entry_set_text( GTK_ENTRY( widget ), "" );

		widget = glade_xml_get_widget( xml, "helperpath" );
		widget = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(widget));
		gtk_entry_set_text( GTK_ENTRY( widget ), "" );

		widget = glade_xml_get_widget( xml, "helpericon" );
		gnome_icon_entry_set_filename( GNOME_ICON_ENTRY( widget ), "");
	}
}

static void screem_preferences_edit_syntax( ScreemApplication *application,
					    GladeXML *xml )
{
	GHashTable *tables;

	tables = screem_application_load_syntax_tables();

	g_hash_table_foreach( tables, 
			      (GHFunc)screem_preferences_build_syntax,
			      xml );
}

static void screem_preferences_build_syntax( const gchar *filename,
					     GObject *table,
					     GladeXML *xml )
{
	GConfClient *client;
	GladeXML *hxml;
	gchar *gladepath;
	GtkWidget *widget;
	GtkWidget *box;
	GList *mime_types;
	const gchar *mime_type;
	GList *list;
	GList *tags;
	GtkTextTag *tag;
	GList *combo_text;
	gchar *type;

	client = gconf_client_get_default();
	gladepath = gconf_client_get_string( client, 
					     "/apps/screem/general/glade_path",
					     NULL );
	hxml = glade_xml_new( gladepath, "highlighting_frame", NULL );
	g_free( gladepath );

	mime_types = (GList*)g_object_get_data( table, "types" );
	mime_type = (const gchar*)mime_types->data;

	tags = (GList*)g_object_get_data( table, "tags" );
	combo_text = NULL;
	widget = glade_xml_get_widget( hxml, "groupcombo" );
	for( list = tags->next; list; list = list->next ) {
		const gchar *name;
		tag = GTK_TEXT_TAG( list->data );

		if( ! g_object_get_data( G_OBJECT( tag ), "included" ) ) {
			g_object_get( G_OBJECT( tag ), "name", &name, NULL );
			combo_text = g_list_prepend( combo_text, 
						     g_strdup( name ) );
		}
	}

	/* setup combo */
	combo_text = g_list_reverse( combo_text );
	gtk_combo_set_popdown_strings( GTK_COMBO( widget ), combo_text );
	g_list_free( combo_text );
	g_object_set_data( G_OBJECT( widget ), "tags", tags );

	/* set first tag as the current one being displayed */
	screem_preferences_tag_name_changed( GTK_COMBO( widget )->entry );

	box = glade_xml_get_widget( xml, "colours_box" );
	widget = glade_xml_get_widget( hxml, "mime_type" );
	type = g_strconcat( "<span weight=\"bold\">", mime_type, ":</span>",
			    NULL );
	gtk_label_set_markup( GTK_LABEL( widget ), type );
	g_free( type );
	widget = glade_xml_get_widget( hxml, "highlighting_frame" );
	gtk_box_pack_start( GTK_BOX( box ), widget, FALSE, TRUE, 4 );

	g_object_set_data( G_OBJECT( widget ), "table", table );

	glade_xml_signal_autoconnect( hxml );
			     
	g_object_unref( client );
}

void screem_preferences_tag_name_changed( GtkWidget *widget )
{
	GladeXML *xml;
	const gchar *group;
	GtkWidget *combo;
	GList *list;
	GList *tags;
	gboolean found;
	GtkTextTag *tag;
	GtkWidget *frame;

	xml = glade_get_widget_tree( widget );
	group = gtk_entry_get_text( GTK_ENTRY( widget ) );

	frame = glade_xml_get_widget( xml, "highlighting_frame" );

	combo = glade_xml_get_widget( xml, "groupcombo" );

	tags = (GList*)g_object_get_data( G_OBJECT( combo ), "tags" );
	
	for( found = FALSE, list = tags->next; list; list = list->next ) {
		const gchar *name;
		tag = GTK_TEXT_TAG( list->data );

		g_object_get( G_OBJECT( tag ), "name", &name, NULL );

		if( ! strcmp( name, group ) ) {
			found = TRUE;
			break;
		}
	}

	if( found )
	{
		const GdkColor *fr;
		const GdkColor *bg;
		GnomeColorPicker *fpick;
		GnomeColorPicker *bpick;
		gboolean usefore;
		gboolean useback;

		GtkTextTag *ftag;
		GtkTextTag *btag;

		g_object_set_data( G_OBJECT( frame ), "blocked",
				   GINT_TO_POINTER( 1 ) );
	
		tag = GTK_TEXT_TAG( list->data );
	
		ftag = GTK_TEXT_TAG( g_object_get_data( G_OBJECT( tag ), "fore_tag" ) );
		btag = GTK_TEXT_TAG( g_object_get_data( G_OBJECT( tag ), "back_tag" ) );

		fpick = GNOME_COLOR_PICKER( glade_xml_get_widget( xml,
								  "fore" ) );
		bpick = GNOME_COLOR_PICKER( glade_xml_get_widget( xml,
								  "back" ) );

		if( ftag ) {
			g_object_get( G_OBJECT( ftag ), "foreground_gdk", &fr, NULL );
			gnome_color_picker_set_i16( fpick, fr->red, fr->green,
						    fr->blue, 0 );
		}
		if( btag ) {
			g_object_get( G_OBJECT( btag ), "background_gdk", &bg, NULL );
			gnome_color_picker_set_i16( bpick, bg->red, bg->green,
						    bg->blue, 0 );
		}

		widget = glade_xml_get_widget( xml, "pattern" );
		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		gtk_entry_set_text( GTK_ENTRY( widget ), "" );

		usefore = FALSE;
		if( ftag ) {
			usefore = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( ftag ),
								      "usefore" ) );
		}
		useback = FALSE;
		if( btag ) {
			useback = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( btag ),
								      "useback" ) );
		}

		widget = glade_xml_get_widget( xml, "usefore" );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
					      usefore );
		widget = glade_xml_get_widget( xml, "useback" );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
					      useback );

		gtk_widget_set_sensitive( GTK_WIDGET( fpick ), usefore );
		gtk_widget_set_sensitive( GTK_WIDGET( bpick ), useback );

		widget = glade_xml_get_widget( xml, "highlighting_frame" );
		g_object_set_data( G_OBJECT( widget ), "tag", tag );

		g_object_set_data( G_OBJECT( frame ), "blocked", NULL );
	}
}

void screem_preferences_syntax_toggled( GtkWidget *widget )
{
	GladeXML *xml;
	GtkWidget *frame;
	GtkWidget *toggle;
	GtkTextTag *tag;
	GtkTextTag *ftag;
	GtkTextTag *btag;
	GtkTextTagTable *table;
	gboolean usefore;
	gboolean useback;

	xmlDocPtr doc;
	xmlNodePtr node;
	xmlNodePtr fore;
	xmlNodePtr back;
	const gchar *filename;

	const gchar *name;

	xml = glade_get_widget_tree( widget );

	frame = glade_xml_get_widget( xml, "highlighting_frame" );

	if( g_object_get_data( G_OBJECT( frame ), "blocked" ) ) {
		return;
	}

	tag = GTK_TEXT_TAG( g_object_get_data( G_OBJECT( frame ), "tag" ) );
	ftag = GTK_TEXT_TAG( g_object_get_data( G_OBJECT( tag ), "fore_tag" ) );
	btag = GTK_TEXT_TAG( g_object_get_data( G_OBJECT( tag ), "back_tag" ) );
	table = GTK_TEXT_TAG_TABLE( g_object_get_data( G_OBJECT( frame ),
						       "table" ) );
	doc = (xmlDocPtr)g_object_get_data( G_OBJECT( table ), "doc" );
	filename = (const gchar*)g_object_get_data( G_OBJECT( table ),
						    "filename" );

	toggle = glade_xml_get_widget( xml, "usefore" );
	usefore = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) );
	toggle = glade_xml_get_widget( xml, "useback" );
	useback = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) );

	g_object_get( G_OBJECT( tag ), "name", &name, NULL );

	/* find <group name="name"> in doc */
	node = xmlDocGetRootElement( doc );
	g_assert( node );
	node = xml_search_child( node, "highlighting" );
	g_assert( node );
	node = node->xmlChildrenNode;
	g_assert( node );
	while( node ) {
		if( ! strcmp( "group", node->name ) ) {
			gchar *group_name;
			
			group_name = xmlGetProp( node, "name" );
			if( ! strcmp( group_name, name ) ) {
				g_free( group_name );
				break;
			}
			g_free( group_name );
		}
		node = node->next;
	}
	g_assert( node );
	fore = xml_search_child( node, "fore" );
	back = xml_search_child( node, "back" );
	xmlSetProp( fore, "use", usefore ? "1" : "0" );
	xmlSetProp( back, "use", useback ? "1" : "0" );

	g_object_set_data( G_OBJECT( ftag ), "usefore", 
			   GINT_TO_POINTER( usefore ) );
	g_object_set_data( G_OBJECT( btag ), "useback", 
			   GINT_TO_POINTER( useback ) );
	
	xmlSaveFile( filename, doc );
}

void screem_preferences_syntax_fore_set( GnomeColorPicker *cp,
					 guint r, guint g, guint b,
					 guint a )
{
	GladeXML *xml;
	GtkTextTag *tag;
	GtkTextTagTable *table;
	GtkWidget *frame;
	gchar *cvalue;

	xmlDocPtr doc;
	xmlNodePtr node;
	xmlNodePtr fore;
	const gchar *filename;
	const gchar *name;

	xml = glade_get_widget_tree( GTK_WIDGET( cp ) );

	frame = glade_xml_get_widget( xml, "highlighting_frame" );

	if( g_object_get_data( G_OBJECT( frame ), "blocked" ) ) {
		return;
	}

	tag = GTK_TEXT_TAG( g_object_get_data( G_OBJECT( frame ), "tag" ) );
	g_object_get( G_OBJECT( tag ), "name", &name, NULL );
	tag = GTK_TEXT_TAG( g_object_get_data( G_OBJECT( tag ), "fore_tag" ) );

	table = GTK_TEXT_TAG_TABLE( g_object_get_data( G_OBJECT( frame ),
						       "table" ) );
	doc = (xmlDocPtr)g_object_get_data( G_OBJECT( table ), "doc" );
	filename = (const gchar*)g_object_get_data( G_OBJECT( table ),
						    "filename" );

	cvalue = g_strdup_printf( "#%.4x%.4x%.4x", r, g, b );
	g_object_set( G_OBJECT( tag ), "foreground", cvalue, NULL );

	/* find <group name="name"> in doc */
	node = xmlDocGetRootElement( doc );
	g_assert( node );
	node = xml_search_child( node, "highlighting" );
	g_assert( node );
	node = node->xmlChildrenNode;
	g_assert( node );
	while( node ) {
		if( ! strcmp( "group", node->name ) ) {
			gchar *group_name;
			
			group_name = xmlGetProp( node, "name" );
			if( ! strcmp( group_name, name ) ) {
				g_free( group_name );
				break;
			}
			g_free( group_name );
		}
		node = node->next;
	}
	g_assert( node );
	fore = xml_search_child( node, "fore" );
	xmlSetProp( fore, "color", cvalue );

	g_free( cvalue );

	xmlSaveFile( filename, doc );
}

void screem_preferences_syntax_back_set( GnomeColorPicker *cp,
					 guint r, guint g, guint b,
					 guint a )
{
	GladeXML *xml;
	GtkTextTagTable *table;
	GtkTextTag *tag;
	GtkWidget *frame;
	gchar *cvalue;

	xmlDocPtr doc;
	xmlNodePtr node;
	xmlNodePtr back;
	const gchar *filename;
	const gchar *name;

	xml = glade_get_widget_tree( GTK_WIDGET( cp ) );

	frame = glade_xml_get_widget( xml, "highlighting_frame" );

	if( g_object_get_data( G_OBJECT( frame ), "blocked" ) ) {
		return;
	}

	tag = GTK_TEXT_TAG( g_object_get_data( G_OBJECT( frame ), "tag" ) );
	g_object_get( G_OBJECT( tag ), "name", &name, NULL );
	tag = GTK_TEXT_TAG( g_object_get_data( G_OBJECT( tag ), "back_tag" ) );


	table = GTK_TEXT_TAG_TABLE( g_object_get_data( G_OBJECT( frame ),
						       "table" ) );
	doc = (xmlDocPtr)g_object_get_data( G_OBJECT( table ), "doc" );
	filename = (const gchar*)g_object_get_data( G_OBJECT( table ),
						    "filename" );

	cvalue = g_strdup_printf( "#%.4x%.4x%.4x", r, g, b );
	g_object_set( G_OBJECT( tag ), "background", cvalue, NULL );

	/* find <group name="name"> in doc */
	node = xmlDocGetRootElement( doc );
	g_assert( node );
	node = xml_search_child( node, "highlighting" );
	g_assert( node );
	node = node->xmlChildrenNode;
	g_assert( node );
	while( node ) {
		if( ! strcmp( "group", node->name ) ) {
			gchar *group_name;
			
			group_name = xmlGetProp( node, "name" );
			if( ! strcmp( group_name, name ) ) {
				g_free( group_name );
				break;
			}
			g_free( group_name );
		}
		node = node->next;
	}
	g_assert( node );
	back = xml_search_child( node, "back" );
	xmlSetProp( back, "color", cvalue );

	g_free( cvalue );

	xmlSaveFile( filename, doc );
}
